#!/usr/bin/env python

import yaml
import os
import sys
import pwd
import grp
import base64
import charmhelpers.core.host
import charmhelpers.core.hookenv
import charmhelpers.fetch
import charmhelpers.contrib.unison

required_pkgs = ['tarmac', 'python-jinja2', 'bzr']

hooks = charmhelpers.core.hookenv.Hooks()
config = {}


def get_cron_root():
    return '/etc/cron.d'


def _get_dir(dir_type):
    service_name = charmhelpers.core.hookenv.service_name()
    return os.path.join('/srv', service_name, dir_type)


def get_var_dir():
    return _get_dir('var')


def get_conf_dir():
    return _get_dir('conf')


def get_log_dir():
    return _get_dir('logs')


def get_ssh_dir():
    return os.path.expanduser('~%s/.ssh' % config['user'])


def get_template_dir():
    return os.path.join(charmhelpers.core.hookenv.charm_dir(), 'templates')


def get_config_template(name):
    from jinja2 import Environment, FileSystemLoader
    template_env = Environment(loader=FileSystemLoader(get_template_dir()))
    return template_env.get_template(name)


def update_tarmac_template(params):
    # Copy.
    allowed = ['log_file', 'lp_username']
    kw = {k:v for k,v in params.iteritems() if k in allowed}
    try:
        kw['branches'] = yaml.safe_load(params['branches'])
    except yaml.YAMLError:
        charmhelpers.core.hookenv.log('Invalid YAML in branches parameter.')
        kw['branches'] = {}
    except KeyError:
        kw['branches'] = {}

    # Validate.
    assert kw.get('log_file'), 'No log location set.'
    kw['log_file'] = os.path.join(get_log_dir(), kw['log_file'])

    # Populate.
    template = get_config_template('tarmac.conf.tmpl')
    with open(os.path.join(get_conf_dir(), 'tarmac.conf'), 'w') as fp:
        fp.write(template.render(**kw))


def update_cron_template(params):
    # Validate.
    assert params.get('user'), 'No user set.'
    assert params.get('schedule'), 'No schedule set.'
    if params.get('environment'):
        try:
            # Convert environment YAML setting to KEY = VALUE pairs, separated
            # by newlines, for insertion into the cron template.
            y = yaml.safe_load(params['environment'])
            new_env = '\n'.join(['%s=%s' % (k,v) for k,v in y.iteritems()])
            params['environment'] = new_env
        except yaml.YAMLError:
            charmhelpers.core.hookenv.log('Invalid YAML in environment parameter.')

    params['var_dir'] = get_var_dir()
    params['conf_dir'] = get_conf_dir()
    params['log_dir'] = get_log_dir()

    # Populate.
    template = get_config_template('cron.tmpl')
    with open(os.path.join(get_cron_root(), 'tarmac'), 'w') as fp:
        fp.write(template.render(**params))


def update_credentials_template(params):
    user = params['user']
    group = params['group']
    template = get_config_template('credentials.tmpl')
    path = os.path.join(get_conf_dir(), 'credentials')
    data = template.render(**params)
    kw = {'owner': user, 'group': group, 'perms': 0600}
    charmhelpers.core.host.write_file(path, data, **kw)


def update_ssh_template(params):
    '''Populate ~/.ssh/config and ~/.ssh/known_hosts'''
    user = params['user']
    group = params['group']
    template = get_config_template('ssh_config.tmpl')
    path = os.path.join(get_ssh_dir(), 'config')
    data = template.render(**params)
    kw = {'owner': user, 'group': group, 'perms': 0600}
    charmhelpers.core.host.write_file(path, data, **kw)

    template = get_config_template('known_hosts.tmpl')
    data = template.render(**params)
    path = os.path.join(get_ssh_dir(), 'known_hosts')
    kw = {'owner': user, 'group': group, 'perms': 0600}
    charmhelpers.core.host.write_file(path, data, **kw)


def setup_ssh(private_key='', public_key=''):
    '''Write the private and public ssh keys to ~/.ssh/id_rsa{,.pub}'''
    user = config['user']
    group = config['group']

    ssh_dir = get_ssh_dir()
    if not os.path.exists(ssh_dir):
        kw = {'owner': user, 'group': group, 'perms': 0700}
        charmhelpers.core.host.mkdir(ssh_dir, **kw)

    if private_key:
        kw = {'owner': user, 'group': group, 'perms': 0400}
        path = os.path.join(ssh_dir, 'id_rsa')
        charmhelpers.core.host.write_file(path, private_key, **kw)

    if public_key:
        kw = {'owner': user, 'group': group, 'perms': 0400}
        path = os.path.join(ssh_dir, 'id_rsa.pub')
        charmhelpers.core.host.write_file(path, public_key, **kw)

    update_ssh_template(config)


@hooks.hook('user-relation-changed')
def user_joined(rel_id=None):
    user = config['user']
    group = config['group']
    kw = {'username': user, 'group': group}
    charmhelpers.core.hookenv.relation_set(relation_id=rel_id, **kw)

@hooks.hook()
@hooks.hook('upgrade-charm')
@hooks.hook('config-changed')
def install():
    # Create user and group.
    charmhelpers.core.hookenv.log('Creating user and group.')
    user = config['user']
    group = config['group']
    password = charmhelpers.core.host.pwgen()
    charmhelpers.core.host.adduser(user, password)
    charmhelpers.core.host.add_user_to_group(user, group)

    # Create directories.
    charmhelpers.core.hookenv.log('Creating directories.')
    for path in (get_var_dir(), get_conf_dir(), get_log_dir()):
        charmhelpers.core.host.mkdir(path, owner=user, group=group, perms=0755)

    # Install packages.
    charmhelpers.core.hookenv.log('Installing packages.')
    charmhelpers.fetch.configure_sources(update=True)
    pkgs = required_pkgs + config.get('packages', '').split(' ')
    charmhelpers.fetch.apt_install(pkgs, fatal=True)

    # Set up ssh.
    private_key = config.get('private_key')
    public_key = config.get('public_key')
    if private_key and public_key:
        private_key = base64.b64decode(private_key)
        public_key = base64.b64decode(public_key)
        setup_ssh(private_key=private_key, public_key=public_key)

    # Log into Launchpad.
    if config.get('lp_username'):
        charmhelpers.core.hookenv.log('Logging into Launchpad.')
        cmd = ['bzr', 'launchpad-login', config['lp_username']]
        charmhelpers.contrib.unison.run_as_user(user, cmd)

    # Populate templates.
    charmhelpers.core.hookenv.log('Populating templates.')
    update_tarmac_template(config)
    update_cron_template(config)
    update_credentials_template(config)


if __name__ == '__main__':
    config = charmhelpers.core.hookenv.config()
    hooks.execute(sys.argv)
