import pwd
import grp
import os
import tempfile
import unittest
import hooks
import mock
import shutil
import yaml
import stat


PATCHES = [
    'charmhelpers.core.hookenv.log',
    'charmhelpers.core.host.log',
    'charmhelpers.fetch.apt_install',
    'charmhelpers.fetch.configure_sources',
    'charmhelpers.core.host.adduser',
    'charmhelpers.core.host.add_user_to_group',
]


class InstallTestCase(unittest.TestCase):
    def setUp(self):
        super(InstallTestCase, self).setUp()
        for spec in PATCHES:
            attr_name = spec.split('.')[-1]
            _m = mock.patch(spec)
            setattr(self, attr_name, _m.start())
            self.addCleanup(_m.stop)

        # Set a temporary {config, log} directory, to avoid writing under /srv.
        for attr_name in ('conf_dir', 'log_dir', 'var_dir'):
            temp_dir = tempfile.mkdtemp()
            setattr(self, attr_name, temp_dir)
            self.addCleanup(shutil.rmtree, temp_dir)
            _m = mock.patch('hooks.get_%s' % attr_name, return_value=temp_dir)
            _m.start()
            self.addCleanup(_m.stop)

        # Set a temporary cron directory, to avoid writing under /etc/cron.d.
        self.cron_root = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, self.cron_root)
        _m = mock.patch('hooks.get_cron_root', return_value=self.cron_root)
        _m.start()
        self.addCleanup(_m.stop)

        # Populate the environment as Juju would.
        env = {'CHARM_DIR': '.', 'JUJU_UNIT_NAME': 'tarmac/0'}
        patch_env = mock.patch.dict('os.environ', env)
        patch_env.start()
        self.addCleanup(patch_env.stop)

        # Populate the config.yaml environment as Juju would.
        cfg = {'log_file': 'foo.log',
               'user': os.environ['USER'],
               'group': grp.getgrgid(os.getgroups()[0]).gr_name,
               'schedule': '*/10 * * * *',
              }
        patch_config = mock.patch.dict('hooks.config', cfg)
        patch_config.start()
        self.addCleanup(patch_config.stop)

    def test_install(self):
        hooks.install()
        self.apt_install.assert_called_with(hooks.required_pkgs, fatal=True)
        user = hooks.config['user']
        group = hooks.config['group']
        self.assertEqual(self.adduser.call_args[0][0], user)
        self.add_user_to_group.assert_called_with(user, group)

        for path in (hooks.get_log_dir(),
                     hooks.get_var_dir(),
                     hooks.get_conf_dir()):
            statbuf = os.stat(path)
            self.assertEqual(statbuf.st_uid, pwd.getpwnam(user).pw_uid)
            self.assertEqual(statbuf.st_gid, grp.getgrnam(group).gr_gid)

    def test_setup_ssh(self):
        temp_dir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, temp_dir)
        ssh_dir = os.path.join(temp_dir, '.ssh')
        _p = mock.patch('hooks.get_ssh_dir', return_value=ssh_dir)
        _p.start()
        self.addCleanup(_p.stop)

        hooks.setup_ssh(private_key='A', public_key='B')

        private_key = os.path.join(ssh_dir, 'id_rsa')
        with open(os.path.join(ssh_dir, 'id_rsa')) as fp:
            self.assertEqual(fp.read(), 'A')

        pub_key = os.path.join(ssh_dir, 'id_rsa.pub')
        with open(pub_key) as fp:
            self.assertEqual(fp.read(), 'B')

        for k in (pub_key, private_key, ssh_dir):
            stat_buf = os.stat(k).st_mode
            self.assertFalse(stat_buf & stat.S_IROTH)
            self.assertFalse(stat_buf & stat.S_IRGRP)
            self.assertTrue(stat_buf & stat.S_IRUSR)
        stat_buf = os.stat(ssh_dir).st_mode
        self.assertTrue(stat_buf & stat.S_IXUSR)


class TestTemplatePopulation(InstallTestCase):

    def test_basic_population(self):
        params = {'log_file': 'foo.log'}
        hooks.update_tarmac_template(params)
        with open(os.path.join(self.conf_dir, 'tarmac.conf')) as fp:
            self.assertIn('log_file = %s/foo.log' % self.log_dir, fp.read())

    def test_branch_population(self):
        params = {'log_file': 'foo.log',
                  'branches': yaml.safe_dump({
                      'lp:foo': {'check_command': '/bin/false'},
                   }),
                 }
        hooks.update_tarmac_template(params)
        with open(os.path.join(self.conf_dir, 'tarmac.conf')) as fp:
            tmpl_data = fp.read()

        self.assertIn('[lp:foo]', tmpl_data)
        self.assertIn('check_command = /bin/false', tmpl_data)

    def test_cron_population(self):
        user = os.environ['USER']
        env = '{ OS_USERNAME: ev, OS_PASSWORD: foo }'
        params = {'user': user, 'schedule': '*/5 * * * *', 'environment': env}
        hooks.update_cron_template(params)
        with open(os.path.join(self.cron_root, 'tarmac')) as fp:
            tmpl_data = fp.read()

        self.assertIn('*/5 * * * *', tmpl_data)
        self.assertIn('TARMAC_CONFIG_HOME=%s' % self.conf_dir, tmpl_data)
        self.assertIn('OS_USERNAME=ev', tmpl_data)
        self.assertIn('OS_PASSWORD=foo', tmpl_data)

    def test_credentials_population(self):
        params = {'oauth_consumer_key': 'A',
                  'oauth_consumer_secret': '',
                  'oauth_access_token': 'B',
                  'oauth_access_secret': 'C',
                  'user': os.environ['USER'],
                  'group': grp.getgrgid(os.getgroups()[0]).gr_name,
                 }
        hooks.update_credentials_template(params)
        with open(os.path.join(self.conf_dir, 'credentials')) as fp:
            tmpl_data = fp.read()
        self.assertIn('consumer_key = A', tmpl_data)
        self.assertIn('consumer_secret = \n', tmpl_data)
        self.assertIn('access_token = B', tmpl_data)
        self.assertIn('access_secret = C', tmpl_data)


if __name__ == '__main__':
    unittest.main()
